package pl.wat.wcy.openmap.rmi.dted;

import java.awt.Dimension;
import java.awt.Image;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.image.BufferedImage;
import java.rmi.RMISecurityManager;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.rmi.server.RemoteObject;
import java.rmi.server.UnicastRemoteObject;
import java.util.ArrayList;
import java.util.StringTokenizer;
import java.util.Vector;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import pl.wat.wcy.openmap.rmi.dted.support.RMIDTEDFrameSubframeInfo;
import pl.wat.wcy.openmap.rmi.dted.support.RemoteCADRG;
import pl.wat.wcy.openmap.rmi.dted.support.RemoteLLXY;
import pl.wat.wcy.openmap.rmi.dted.support.RemoteRasterImage;


import com.bbn.openmap.layer.dted.DTEDCacheManager;
import com.bbn.openmap.layer.dted.DTEDFrameSubframe;
import com.bbn.openmap.layer.dted.DTEDFrameSubframeInfo;
import com.bbn.openmap.omGraphics.OMGraphic;
import com.bbn.openmap.omGraphics.OMGraphicList;
import com.bbn.openmap.omGraphics.OMRaster;
import com.bbn.openmap.omGraphics.OMRasterObject;
import com.bbn.openmap.proj.CADRG;
import com.bbn.openmap.proj.EqualArc;
import com.bbn.openmap.util.Debug;

/**
 * @author Mariusz Chmielewski
 * Cyebrnetics Faculty
 * Military University of Technology
 * mchmielewski@wat.edu.pl  
 *
 */

public class RMIDTEDServer extends RemoteObject implements RMIDTEDRemote {

	protected String[] dtedpaths = new String[] { "/mnt/cdrom/dted",
			"/mnt/disk/dted" };

	protected String[] dted2paths = new String[] { "/mnt/cdrom/dted_level2",
			"/mnt/disk/dted_level2" };

	protected int ncolors = 216;

	protected int opaque = 255;

	public DTEDCacheManager cache_manager;

	protected int dtedLevel = DTEDFrameSubframe.LEVEL_0;

	protected int viewType = DTEDFrameSubframe.COLOREDSHADING;

	protected int bandHeight = DTEDFrameSubframe.DEFAULT_BANDHEIGHT;

	protected int slopeAdjust = DTEDFrameSubframe.DEFAULT_SLOPE_ADJUST;

	protected float minScale = 20000000f;

	/** 1099, the default number for RMI registry. */
	public final static int DEFAULT_RMIREGISTRY_PORT = 1099;

	/** 7000, the default port number for RMIDTEDServer. */
	public final static int DEFAULT_RMIDTEDSERVER_PORT = 7000;

	/** localhost 127.0.0.1, the default RMI registry server IP/NAME. */
	public final static String DEFAULT_RMIREGISTRY_MACHINE = "127.0.0.1";

	/** Port number for the RMI reristry lookup */
	int RMI_port = DEFAULT_RMIREGISTRY_PORT;

	/** Port number for the service */
	int RMIDTEDServer_port = DEFAULT_RMIDTEDSERVER_PORT;

	/** An RMI Registry object. */
	Registry registry;

	String serverHostName = DEFAULT_RMIREGISTRY_MACHINE;

	/** The RMIRpfServer service name. */
	private String serverName = "RMIDTEDServer";

	protected void init() {
		cache_manager = new DTEDCacheManager(dtedpaths, dted2paths, ncolors,
				opaque);
		DTEDFrameSubframeInfo dfsi = new DTEDFrameSubframeInfo(viewType,
				bandHeight, dtedLevel, slopeAdjust);
		dfsi.colorModel = OMRasterObject.COLORMODEL_INDEXED;
		cache_manager.setSubframeInfo(dfsi);

	}

	/** forwarding the request to local DTEDCacheManager - getElevation */
	public int getElevation(float lat, float lon) throws RemoteException {
		System.out.println("...invoked getElevation()");
		return cache_manager.getElevation(lat, lon);
	}

	/** forwarding the request to local DTEDCacheManager - setSubframeInfo */
	public void setSubframeInfo(RMIDTEDFrameSubframeInfo dfsi)
			throws RemoteException {
		System.out.println("...invoked setSubframeInfo()");
		cache_manager.setSubframeInfo(dfsi.transform());
	}

	/** forwarding the request to local DTEDCacheManager - getSubframeInfo */
	public RMIDTEDFrameSubframeInfo getSubframeInfo() throws RemoteException {
		System.out.println("...invoked getSubframeInfo()");
		return new RMIDTEDFrameSubframeInfo(cache_manager.getSubframeInfo());
	}

	/** forwarding the request to local DTEDCacheManager - setDtedDirPaths */
	public void setDtedDirPaths(String[] paths) throws RemoteException {
		System.out.println("...invoked setDtedDirPaths()");
		cache_manager.setDtedDirPaths(paths);
	}

	/** forwarding the request to local DTEDCacheManager - setDtedDirPaths */
	public void setDtedDirPaths(String[] paths, String[] paths2)
			throws RemoteException {
		System.out.println("...invoked setDtedDirPaths()");
		cache_manager.setDtedDirPaths(paths, paths2);
	}

	/** forwarding the request to local DTEDCacheManager - setCacheSize */
	public void setCacheSize(int size) throws RemoteException {
		System.out.println("...invoked setCacheSize()");
		cache_manager.setCacheSize(size);
	}

	/** forwarding the request to local DTEDCacheManager - setNumColors */
	public void setNumColors(int numberOfColors) throws RemoteException {
		System.out.println("...invoked setNumColors()");
		cache_manager.setNumColors(numberOfColors);
	}

	/** forwarding the request to local DTEDCacheManager - setOpaqueness */
	public void setOpaqueness(int setting) throws RemoteException {
		System.out.println("...invoked setOpaqueness()");
		cache_manager.setOpaqueness(setting);
	}

	/** forwarding the request to local DTEDCacheManager - getRectangle */
	public ArrayList getRectangle(RemoteCADRG proj) throws RemoteException{
		System.out.println("...invoked getRectangle() based on CADRG projection");
		return transform(cache_manager.getRectangle(proj.transform()));
	}

	/** forwarding the request to local DTEDCacheManager - getRectangle */
	public ArrayList getRectangle(RemoteLLXY proj) throws RemoteException{
		System.out.println("...invoked getRectangle() based on LLXY projection");
		return transform(cache_manager.getRectangle(proj.transform()));
	}

	private ArrayList transform(OMGraphicList om) {
		ArrayList res = new ArrayList();
		for (int i = 0; i < om.size(); i++) {
			res.add(new RemoteRasterImage((OMRaster) om.getOMGraphicAt(i)));
		}
		return res;
	}

	public void printHelp() {
		System.err
				.println("usage: java [java JVM args] <server class> [server args]");
		System.err.println("");
		System.err.println("       Java Args:");
		System.err
				.println("       -mx<NUM>m               Set max Java heap in Megs");
		System.err.println("");
		System.err.println("       Server Args:");
		System.err
				.println("       -rmiport <port for RMI registry>			Port of RMI registry");
		System.err
				.println("       -rmihost <name host for RMI registry>     	RMI Registry Host");
		System.err
				.println("       -serverport <port for the RMIDTEDServer>   Service port");
		System.err
				.println("       -dtedpaths \"<path1> ...\"      			Path to search for DTED data");
		System.err
				.println("       -dted2paths \"<path1> ...\"     			Path to search for DTED level2 data");
		System.err
				.println("       -level <0|1|2>                  			DTED level (default is 0)");
	}

	private String[] getPaths(String str) {
		StringTokenizer tok = new StringTokenizer(str);
		int len = tok.countTokens();
		String[] paths = new String[len];
		for (int j = 0; j < len; j++) {
			paths[j] = tok.nextToken();
		}
		return paths;
	}

	public void parseArgs(String[] args) {
		for (int i = 0; i < args.length; i++) {
			if (args[i].equalsIgnoreCase("-rmiport")) {
				RMI_port = Integer.parseInt(args[++i]);
			} else if (args[i].equalsIgnoreCase("-rmihost")) {
				serverHostName = args[++i];
			} else if (args[i].equalsIgnoreCase("-serverport")) {
				RMIDTEDServer_port = Integer.parseInt(args[++i]);
			} else if (args[i].equalsIgnoreCase("-help")) {
				printHelp();
			} else if (args[i].equalsIgnoreCase("-h")) {
				printHelp();
			} else if (args[i].equalsIgnoreCase("-dtedpaths")) {
				dtedpaths = getPaths(args[++i]);
			} else if (args[i].equalsIgnoreCase("-dted2paths")) {
				dted2paths = getPaths(args[++i]);
			} else if (args[i].equalsIgnoreCase("-level")) {
				switch (Integer.parseInt(args[++i])) {
				case 1:
					System.out.println("Setting level 1");
					dtedLevel = DTEDFrameSubframe.LEVEL_1;
					break;
				case 2:
					System.out.println("Setting level 2");
					dtedLevel = DTEDFrameSubframe.LEVEL_2;
					break;
				case 0:
				default:
					System.out.println("Setting level 0");
					dtedLevel = DTEDFrameSubframe.LEVEL_0;
					break;
				}
			}
		}

	}

	/**
	 * @param args
	 * @throws RemoteException
	 */
	public void start(String[] args) throws RemoteException {

		/*
		 * if (args != null) { parseArgs(args); }
		 */

		try {
			registry = LocateRegistry.getRegistry(serverHostName, RMI_port);
		} catch (Exception e) {
			e.printStackTrace();
		}
		Debug.message("RMIDTEDServer",
				"RMIDTEDServer: Exporting remite server object...");
		UnicastRemoteObject.exportObject(this, RMIDTEDServer_port);
		registry.rebind(serverName, this);
		Debug.message("RMIDTEDServer", "RMIDTEDServer: Service binded...");

	}

	public static void main(String[] args) {
		Debug.init(System.getProperties());
		// Set security manager
		Debug.message("RMIDTEDServer",
				"RMIDTEDServer: Setting the security manager.");

		System.setSecurityManager(new RMISecurityManager());

		try {
			// Create the specialist server
			RMIDTEDServer srv = new RMIDTEDServer();
			Debug.message("RMIDTEDServer",
					"RMIDTEDServer: Initializing server...");
			srv.parseArgs(args);
			srv.init();
			Debug.message("RMIDTEDServer",
					"RMIDTEDServer: Preparing DTED data...");
			srv.start(args);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

}
