package pl.wat.wcy.openmap.rmi.dted.support;

import java.io.Serializable;

import com.bbn.openmap.layer.dted.DTEDFrameSubframeInfo;
import com.bbn.openmap.omGraphics.OMRasterObject;

/**
 * @author Mariusz Chmielewski
 * Cyebrnetics Faculty
 * Military University of Technology
 * mchmielewski@wat.edu.pl  
 *
 */

public class RMIDTEDFrameSubframeInfo implements Serializable {

	/**
     * The type of shading to use in the image. DTEDFrameSubframe has
     * the different types of possibilities.
     */
    public int viewType;
    /**
     * The distance (elevation) range that each color section of the
     * band viewtype represents.
     */
    public int bandHeight;
    /** The DTED level of the data. */
    public int dtedLevel;
    /**
     * A user adjusted setting for contrast for the slope shading.
     * Some areas of the earth require different settings, for more or
     * less detail.
     */
    public int slopeAdjust;
    /**
     * The degrees/pixel of the image in the x direction, derived from
     * the scale of the projection.
     */
    public double xPixInterval;
    /**
     * The degrees/pixel of the image in the y direction, derived from
     * the scale of the projection.
     */
    public double yPixInterval;
    /**
     * x number of the subframe within the DTEDFrame, from the left
     * side of the frame (east).
     */
    public int subx;
    /**
     * y number of the subframe within the DTEDFrame, from the bottom
     * side of the frame (south).
     */
    public int suby;
    /** Longitude of the upper left corner of the subframe image. */
    public float lon;
    /** Latitude of the upper left corner of the subframe image. */
    public float lat;
    /** height of the subframe image. */
    public int height;
    /** Width of the subframe image. */
    public int width;
    /**
     * The colormodel of the images. Use
     * OMRasterObject.COLORMODEL_DIRECT (default) or
     * OMRasterObject.COLORMODEL_INDEXED
     */
    public int colorModel = OMRasterObject.COLORMODEL_DIRECT;



	public RMIDTEDFrameSubframeInfo() {
		
	}

	public RMIDTEDFrameSubframeInfo(DTEDFrameSubframeInfo source) {
		viewType = source.viewType;
	    bandHeight = source.bandHeight;
	    dtedLevel = source.dtedLevel;
	    slopeAdjust = source.slopeAdjust;
	    xPixInterval = source.xPixInterval;
	    yPixInterval = source.yPixInterval;
	    subx = source.subx;
	    suby = source.suby;
	    lon = source.lon;
	    lat = source.lat;
	    height = source.height;
	    width = source.width;
	    colorModel = source.colorModel;
	}

	public DTEDFrameSubframeInfo transform (){
		DTEDFrameSubframeInfo res = new DTEDFrameSubframeInfo(height,width,lon,lat,subx,suby,xPixInterval,yPixInterval,viewType,bandHeight,dtedLevel,slopeAdjust);
		res.colorModel = colorModel;
		return res;
	}
}
